library(tuneR)
library(gplots)
library(HMM)

source("getFunctions.R")

segments <- getSegments( "szamok.wav" )

sounds <- list()

sounds[[1]] <- getCepstra( "egy.wav" )
sounds[[2]] <- getCepstra( "ketto.wav" )
sounds[[3]] <- getCepstra( "harom.wav" )
sounds[[4]] <- getCepstra( "negy.wav" )
sounds[[5]] <- getCepstra( "ot.wav" )
sounds[[6]] <- getCepstra( "hat.wav" )
sounds[[7]] <- getCepstra( "het.wav" )
sounds[[8]] <- getCepstra( "nyolc.wav" )
sounds[[9]] <- getCepstra( "kilenc.wav" )

c10 <- cbind( sounds[[1]], sounds[[2]], sounds[[3]], sounds[[4]], sounds[[5]], sounds[[6]], sounds[[7]], sounds[[8]], sounds[[9]] )

km <- kmeans( t(c10), 32 )

hmm.init <- initHMM( States = c("A","B","C"),
                     Symbols = paste0("C",as.character(1:32)),
                     startProbs = c(0.7,0.2,0.1),
                     transProbs = matrix( c( 0.7,0.25,0.05,
                                             0.0, 0.7, 0.3,
                                             0.05,0.05,0.9), 3, byrow = T) )

hmm.bw <- list()

for( i in 1:9 )
{
  hmm.bw[[i]] <- baumWelch( hmm.init, getCodebook( km$centers, sounds[[i]] ), pseudoCount = 1 )
}



for( i in 1:length(segments) )
{
  max.prob <- 0
  max.index <- 0
  for( j in 1:length(sounds) )
  {
    fw.table <- forward( hmm.bw[[j]]$hmm, getCodebook( km$centers, getCepstraFromSound( segments[[i]] ) ))
    prob <- sum(exp(fw.table[,ncol(fw.table)])) 
    
    if( max.index == 0 | prob > max.prob )
    {
      max.index <- j
      max.prob <- prob
    }
  }
  
  print( paste0( "Sound: ", i, ", HMM: ", max.index ) )
}

