getCepstra <- function( wavfile )
{
  egy <- readWave( wavfile )
  
  egy.left <- egy@left
  egy.left <- egy.left / 2^(egy@bit -1)
  
  pspectrum <- powspec(egy@left, egy@samp.rate)
  aspectrum <- audspec(pspectrum, egy@samp.rate)
  lpcas <- dolpc(aspectrum$aspectrum, 20)
  cepstra <- lpc2cep(lpcas)
  
  return( cepstra[2:21,] )
}

getCepstraFromSound <- function( sound )
{
  egy <- sound
  
  egy.left <- egy@left
  egy.left <- egy.left / 2^(egy@bit -1)
  
  pspectrum <- powspec(egy@left, egy@samp.rate)
  aspectrum <- audspec(pspectrum, egy@samp.rate)
  lpcas <- dolpc(aspectrum$aspectrum, 20)
  cepstra <- lpc2cep(lpcas)
  
  return( cepstra[2:21,] )
}

getDistance <- function( x, y )
{
  return( as.numeric( dist(rbind(x,y)) ) )
}

getCodebook <- function( centers, cepstra )
{
  codebook <- c()
  for( i in 1:ncol(cepstra) )
  {
    d <- 0
    m <- 0
    for( j in 1:nrow(centers) )
    {
      s <- getDistance( cepstra[,i], centers[j,] )
      if( m == 0 | s < d )
      {
        m <- j
        d <- s
      }
    }
    
    codebook <- c( codebook, paste0("C",m) )
  }
  
  return( codebook )
}

getSegments <- function( wavfile )
{
  egy <- readWave( wavfile )
  
  egy.left <- egy@left
  egy.left <- egy.left / 2^(egy@bit -1)
  
  mean.rms <- mean( egy.left^2 )
  
  win.length <- egy@samp.rate / 100
  win.overlap <- win.length * 0.25
  
  segments.base <- c()
  
  for( i in 1:ceiling(length(egy.left)/win.length) )
  {
    start <- (i-1)*win.length + 1
    end <- min( i*win.length, length(egy.left) )
    win.rms <- mean( egy.left[start:end]^2 )
    
    segments.base <- c( segments.base, ifelse( win.rms < mean.rms / 5.5, 0, 1 ) )
  }
  
  start.index <- 0
  end.index <- 0
  in.segment <- F
  segment.index <- 1
  ll <- 25
 
  segments <- list()
  
  for( i in 1:length(segments.base) )
  {
    value <- mean(segments.base[i:min(length(segments.base),(i+ll))])
    # print( paste( "i=", i, ", mean=", value ))
    
    if( value >= 1/(ll+1) & !in.segment )
    {
      start.index <- min( which( segments.base[i:min(length(segments.base),(i+ll))] == 1 ) ) + i - 1
      in.segment <- T
    } else if( value < 1/(ll+1) & in.segment ) {
      indices <- which( segments.base[i:min(length(segments.base),(i+ll))] == 1 )
      end.index <- ifelse( length(indices) > 0 , max( indices ) + i - 1, i )
      in.segment <- F
      
      if( end.index - start.index > ll )
      {
        segments[[segment.index]] <- readWave( wavfile, from = start.index / 100, to = end.index / 100, units = "seconds" )
      
        # print( paste( " from =", start.index / 100, "to =", end.index / 100 ))
      
        segment.index <- segment.index + 1
      }
    }
  }
  
  return( segments )
}